var dmtPreferencesService = null;
var dmtPrevSelectedCount = 0;
var dmtPrevSelectedItem = null;
var dmtDownloadManager = null;
var dmtDownloadListener = null;
var dmtSavedDownloadsView = null;
var dmtProjectName = "Download Manager Tweak";
var dmtReloadButtonLabel = "";
var dmtLaunchButtonLabel = "";
var dmtShowButtonLabel = "";
var dmtDeleteFileButtonLabel = "";
var dmtCleanUpButtonLabel = "";
var dmtCleanUpSepFlex = 0;
var dmtButtonsWidth = 0;
var dmtWindowWidth = 0;
var dmtResizeTimeoutId = 0;

////////////////////////////////////////////////////////////////////////////////
// This event listener is called whenever the selection changes for the
// download list. Need to selectively enable/disable the command bar buttons
// and fill in the infoBar data.
function dmtOnDownloadSelect(aEvent)
{
  var dlState = 0;

  // Make sure selection changed so we don't redraw too often.
  if ((gDownloadsView.selectedCount != dmtPrevSelectedCount) ||
      (gDownloadsView.selectedItem != dmtPrevSelectedItem))
  {
    dmtPrevSelectedCount = gDownloadsView.selectedCount;
    dmtPrevSelectedItem = gDownloadsView.selectedItem;

    // Get parameter for dmtUpdateToolInfoBar().
    if ((gDownloadsView.selectedCount > 0) && gDownloadsView.selectedItem)
    {
      dlState = parseInt(gDownloadsView.selectedItem.getAttribute("state"));
    }
    dmtUpdateToolInfoBar(dlState);
  }
}

////////////////////////////////////////////////////////////////////////////////
// Updates all the toolbar and infobar parts which change when different
// downloads are selected or the selected changes state.
function dmtUpdateToolInfoBar(dlState)
{
  // Create variables, initialized to case of no download selected.
  var reloadDisable     = true;
  var launchDisable     = true;
  var showDisable       = true;
  var deleteFileDisable = true;
  var cleanUpDisable    = true;
  var from = "";
  var to   = "";
  var date = "";

  if ((gDownloadsView.selectedCount > 0) && gDownloadsView.selectedItem)
  {
    var fileExists = false;
    var dirExists = false;
    var dateObj = new Date(parseInt(gDownloadsView.selectedItem.getAttribute("endTime")));

    // Fill in the infoBar data according to the selected download.
    from = gDownloadsView.selectedItem.getAttribute("uri");
    to   = gDownloadsView.selectedItem.getAttribute("path");
    date = dateObj.toLocaleString();


    // Enable buttons for the selected download according to the state of
    // that download.
    switch (dlState) {
    case nsIDM.DOWNLOAD_FINISHED:
      // Check to see if the file for selected download actually exists. It may
      // have been deleted or the download didn't complete.
      try {
        var fileName = gDownloadsView.selectedItem.getAttribute("file");
        var fileObj = getLocalFileFromNativePathOrUrl(fileName);
        fileExists = fileObj.exists();
        dirExists = fileObj.parent.exists();
      } catch (e) {
        fileExists = false
      }

      // Only allow some buttons if a file exists. This same criteria is used
      // by the command handlers so disabling these doesn't really prevent the
      // command.
      if (fileExists)
      {
        launchDisable = false;
        deleteFileDisable = false;
      }
      else
        reloadDisable = false;
      if (dirExists)
        showDisable = false;
      cleanUpDisable = false;
      break;
    case nsIDM.DOWNLOAD_FAILED:
    case nsIDM.DOWNLOAD_CANCELED:
      reloadDisable = false;
      cleanUpDisable = false;
      break;
    case nsIDM.DOWNLOAD_PAUSED:
      cleanUpDisable = false;
      break;
    case nsIDM.DOWNLOAD_NOTSTARTED:
    case nsIDM.DOWNLOAD_DOWNLOADING:
    case nsIDM.DOWNLOAD_QUEUED:
    case nsIDM.DOWNLOAD_BLOCKED_PARENTAL:
    case nsIDM.DOWNLOAD_SCANNING:
    case nsIDM.DOWNLOAD_DIRTY:
    case nsIDM.DOWNLOAD_BLOCKED_POLICY:
    default:
      // All buttons remain disabled.
      break;
    }
  }

  // Display either reload or launch button and not both.
  var hideLaunchButton = !dmtPreferencesService.getBoolPref("extensions.downloadmgr.buttonLaunch");
  if (!hideLaunchButton) {
    document.getElementById("dmtReloadButton").hidden      = reloadDisable;
    document.getElementById("dmtReloadButton2").hidden     = reloadDisable;
    document.getElementById("dmtReloadSep").hidden         = reloadDisable;
    document.getElementById("dmtReloadSep2").hidden        = reloadDisable;
    document.getElementById("dmtLaunchButton").hidden      = !reloadDisable;
    document.getElementById("dmtLaunchButton2").hidden     = !reloadDisable;
    document.getElementById("dmtLaunchSep").hidden         = !reloadDisable;
    document.getElementById("dmtLaunchSep2").hidden        = !reloadDisable;
  }

  // Set the button disables as calculated.
  document.getElementById("dmtReloadButton").disabled      = reloadDisable;
  document.getElementById("dmtReloadButton2").disabled     = reloadDisable;
  document.getElementById("dmtLaunchButton").disabled      = launchDisable;
  document.getElementById("dmtLaunchButton2").disabled     = launchDisable;
  document.getElementById("dmtShowButton").disabled        = showDisable;
  document.getElementById("dmtShowButton2").disabled       = showDisable;
  document.getElementById("dmtDeleteFileButton").disabled  = deleteFileDisable;
  document.getElementById("dmtDeleteFileButton2").disabled = deleteFileDisable;
  document.getElementById("dmtCleanUpButton").disabled     = cleanUpDisable;
  document.getElementById("dmtCleanUpButton2").disabled    = cleanUpDisable;

  // Fill in the infoBar with generated data.
  document.getElementById("dmtDownloadFrom").value  = from;
  document.getElementById("dmtDownloadFrom2").value = from;
  document.getElementById("dmtDownloadTo").value    = to;
  document.getElementById("dmtDownloadTo2").value   = to;
  document.getElementById("dmtDownloadDate").value  = date;
  document.getElementById("dmtDownloadDate2").value = date;
}

////////////////////////////////////////////////////////////////////////////////
// gDownloadsView changes on occasion, for example whenever buildDownloadList()
// is called. That results in the event listener no longer generating select
// events. Need to remove the listener for the old view and and add one for the
// new view. This is only done if gDownloadsView changes.
function dmtOnDownloadClick(aEvent)
{
  if (dmtSavedDownloadsView != gDownloadsView)
  {
    dmtSavedDownloadsView.removeEventListener("select", dmtOnDownloadSelect, false);
    gDownloadsView.addEventListener("select", dmtOnDownloadSelect, false);
    dmtSavedDownloadsView = gDownloadsView;
    dmtOnDownloadSelect(aEvent);
  }
}

////////////////////////////////////////////////////////////////////////////////
// Called whenever the window changes size. If the window width crosses a
// threshold then the buttons will need to be redrawn to fit.
function dmtOnDownloadResize(aEvent)
{
  var windowWidth = dmtDownloadManager.boxObject.width;

  // Only do this if width changed. Note that width starts as 0 so this always
  // runs the first time.
  if (windowWidth != dmtWindowWidth)
  {
    var actionButtons = ["dmtReloadButton",     "dmtReloadButton2",
                         "dmtLaunchButton",     "dmtLaunchButton2",
                         "dmtShowButton",       "dmtShowButton2",
                         "dmtDeleteFileButton", "dmtDeleteFileButton2",
                         "dmtCleanUpButton",    "dmtCleanUpButton2"]

    dmtWindowWidth = windowWidth;
    try {
      if (windowWidth < dmtButtonsWidth + 80)
      {
        // Download Manager is a side bar or a narrow window.
        for (i = 0; i < actionButtons.length; i++)
          document.getElementById(actionButtons[i]).removeAttribute("label");
        document.getElementById("dmtCleanUpSep").flex       = 0;
        document.getElementById("dmtCleanUpSep2").flex      = 0;
        document.getElementById("dmtSearchSpacerSB").hidden = false;
      }
      else
      {
        // Download Manager is a tab or wide window.
        document.getElementById("dmtReloadButton").label      = dmtReloadButtonLabel;
        document.getElementById("dmtReloadButton2").label     = dmtReloadButtonLabel;
        document.getElementById("dmtLaunchButton").label      = dmtLaunchButtonLabel;
        document.getElementById("dmtLaunchButton2").label     = dmtLaunchButtonLabel;
        document.getElementById("dmtShowButton").label        = dmtShowButtonLabel;
        document.getElementById("dmtShowButton2").label       = dmtShowButtonLabel;
        document.getElementById("dmtDeleteFileButton").label  = dmtDeleteFileButtonLabel;
        document.getElementById("dmtDeleteFileButton2").label = dmtDeleteFileButtonLabel;
        document.getElementById("dmtCleanUpButton").label     = dmtCleanUpButtonLabel;
        document.getElementById("dmtCleanUpButton2").label    = dmtCleanUpButtonLabel;
        document.getElementById("dmtCleanUpSep").flex         = dmtCleanUpSepFlex;
        document.getElementById("dmtCleanUpSep2").flex        = dmtCleanUpSepFlex;
        document.getElementById("dmtSearchSpacerSB").hidden   = true;
      }
    } catch (e) {}
  }
}

////////////////////////////////////////////////////////////////////////////////
// Hides and shows window elements depending on the preferences set by the user.
// Can be called at startup or whenever the preferences are changed.
function dmtHideByPreference()
{
  var unhideList = ["dmtReloadButton",     "dmtReloadButton2",
                    "dmtReloadSep",        "dmtReloadSep2",
                    "dmtLaunchButton",     "dmtLaunchButton2",
                    "dmtLaunchSep",        "dmtLaunchSep2",
                    "dmtShowButton",       "dmtShowButton2",
                    "dmtShowSep",          "dmtShowSep2",
                    "dmtDeleteFileButton", "dmtDeleteFileButton2",
                    "dmtDeleteFileSep",    "dmtDeleteFileSep2",
                    "dmtCleanUpButton",    "dmtCleanUpButton2",
                    "dmtOptionsButton",    "dmtOptionsButton2",
                    "dmtTopToolBar",       "dmtBottomToolBar",
                    "dmtDownloadInfoBar",  "dmtDownloadInfoBar2"]

  try {
    // First unhide everything to match original setting in .xul.
    for (i = 0; i < unhideList.length; i++)
      document.getElementById(unhideList[i]).hidden = false;
  } catch (e) {}

  try {
    var showOnBottom = dmtPreferencesService.getBoolPref("extensions.downloadmgr.bottomToolBar");
    if (showOnBottom)
    {
      document.getElementById("dmtTopToolBar").hidden = true;
      document.getElementById("dmtDownloadInfoBar").hidden = true;
    }
    else
    {
      document.getElementById("dmtBottomToolBar").hidden = true;
      document.getElementById("dmtDownloadInfoBar2").hidden = true;
    }

    // Always hide these now, maybe unhidden later?
    document.getElementById("dmtReloadButton").hidden  = true;
    document.getElementById("dmtReloadButton2").hidden = true;
    document.getElementById("dmtReloadSep").hidden  = true;
    document.getElementById("dmtReloadSep2").hidden = true;

    var hideLaunchButton = !dmtPreferencesService.getBoolPref("extensions.downloadmgr.buttonLaunch");
    if (hideLaunchButton) {
      document.getElementById("dmtLaunchButton").hidden  = true;
      document.getElementById("dmtLaunchButton2").hidden = true;
      document.getElementById("dmtLaunchSep").hidden  = true;
      document.getElementById("dmtLaunchSep2").hidden = true;
    }

    var hideShowButton = !dmtPreferencesService.getBoolPref("extensions.downloadmgr.buttonShow");
    if (hideShowButton) {
      document.getElementById("dmtShowButton").hidden  = true;
      document.getElementById("dmtShowButton2").hidden = true;
      document.getElementById("dmtShowSep").hidden  = true;
      document.getElementById("dmtShowSep2").hidden = true;
    }

    var hideDeleteFileButton = !dmtPreferencesService.getBoolPref("extensions.downloadmgr.buttonDelete");
    if (hideDeleteFileButton) {
      document.getElementById("dmtDeleteFileButton").hidden  = true;
      document.getElementById("dmtDeleteFileButton2").hidden = true;
      document.getElementById("dmtDeleteFileSep").hidden  = true;
      document.getElementById("dmtDeleteFileSep2").hidden = true;
    }

    var hideCleanUpButton = !dmtPreferencesService.getBoolPref("extensions.downloadmgr.buttonClean");
    if (hideCleanUpButton) {
      document.getElementById("dmtCleanUpButton").hidden  = true;
      document.getElementById("dmtCleanUpButton2").hidden = true;
    }

    var hideOptionsButton = !dmtPreferencesService.getBoolPref("extensions.downloadmgr.buttonOptions");
    if (hideOptionsButton) {
      document.getElementById("dmtOptionsButton").hidden  = true;
      document.getElementById("dmtOptionsButton2").hidden = true;
    }

    var hideInfoBar = !dmtPreferencesService.getBoolPref("extensions.downloadmgr.showInfoBar");
    if (hideInfoBar) {
      document.getElementById("dmtDownloadInfoBar").hidden  = true;
      document.getElementById("dmtDownloadInfoBar2").hidden = true;
    }
  } catch (e) {}
}

////////////////////////////////////////////////////////////////////////////////
// This observer is called whenever preference in the "extensions.downloadmgr."
// branch changes.
var dmtPrefObserver = {
  observe: function (aSubject, aTopic, aData)
  {
    try {
      if (aTopic == "nsPref:changed")
      {
        if (aData == "extensions.downloadmgr.windowType")
        {
          // Need to close window with old type and then open with new type.
          // Unfortunately I don't think this can be done here.
          //alert("DMT observe: window type changed");
        }
        else
        {
          // Just redraw existing window.
          dmtHideByPreference();
        }
      }
    } catch (e) {}
  }
};

////////////////////////////////////////////////////////////////////////////////
// Setup the extended features added by DMT and then startup the original
// download manger being tweaked.
function dmtStartup()
{
  dmtPreferencesService = Components.classes["@mozilla.org/preferences-service;1"].
                          getService(Components.interfaces.nsIPrefService).getBranch("");
  dmtDownloadManager = document.getElementById("downloadManager");

  // Get some values that are available in the .XUL file, but not easily
  // available here.
  try {
    dmtProjectName           = document.getElementById("dmtDummyLabel").value;
    dmtReloadButtonLabel     = document.getElementById("dmtReloadButton").label;
    dmtLaunchButtonLabel     = document.getElementById("dmtLaunchButton").label;
    dmtShowButtonLabel       = document.getElementById("dmtShowButton").label;
    dmtDeleteFileButtonLabel = document.getElementById("dmtDeleteFileButton").label;
    dmtCleanUpButtonLabel    = document.getElementById("dmtCleanUpButton").label;
    dmtCleanUpSepFlex        = document.getElementById("dmtCleanUpSep").flex;

    // This is used to decide if the window is narrow or wide. Making it
    // variable supports other languages and fonts. Because reload and launch
    // alternate, use the larger of the two and not both.
    if ((document.getElementById("dmtReloadButton").boxObject.width +
         document.getElementById("dmtReloadSep").boxObject.width) >
        (document.getElementById("dmtLaunchButton").boxObject.width +
         document.getElementById("dmtLaunchSep").boxObject.width))
    {
      dmtButtonsWidth =
          document.getElementById("dmtReloadButton").boxObject.width +
          document.getElementById("dmtReloadSep").boxObject.width;
    }
    else
    {
      dmtButtonsWidth =
          document.getElementById("dmtLaunchButton").boxObject.width +
          document.getElementById("dmtLaunchSep").boxObject.width;
    }
    dmtButtonsWidth = dmtButtonsWidth +
          document.getElementById("dmtShowButton").boxObject.width +
          document.getElementById("dmtShowSep").boxObject.width +
          document.getElementById("dmtDeleteFileButton").boxObject.width +
          document.getElementById("dmtDeleteFileSep").boxObject.width +
          document.getElementById("dmtCleanUpButton").boxObject.width +
          document.getElementById("dmtOptionsButton").boxObject.width;

    var hideDmtTheme = dmtPreferencesService.getBoolPref("extensions.downloadmgr.hideDmtTheme");
    if (hideDmtTheme)
    {
      // User wants to use their own theme. Disable the CCS rules for the DMT
      // "theme". These are limited to changes of the underlying download view
      // and not the new to DMT CSS rules.
      for (var i = 0; i < document.styleSheets.length; i++)
      {
        if (document.styleSheets[i].href == 'chrome://downloadmgr/skin/dmtThemeSkin.css')
        {
          // Found the rules for the DMT theme, delete them all.
          for (var j = document.styleSheets[i].cssRules.length - 1; j >= 0; j--)
          {
            document.styleSheets[i].deleteRule(0);
          }
        }
      }
    }
  } catch (e) {alert("DMT Startup() failed:\n" + e.name + "\n" + e.message);}

  dmtOnDownloadResize(null);
  dmtHideByPreference();

  // Startup() is inside mozapps/content/downloads/downloads.js, the download
  // manager built into FireFox. For some reason, DMT causes it to fail by
  // trying to access "window.arguments[1]" when such an attribute doesn't
  // exist. Creating the attribute, calling Statup() and then deleting the
  // attribute will cause it to run without errors.
  try {
    if (window.arguments)
      Startup();
    else
    {
      window.arguments = [0, 0];
      Startup();
      delete window.arguments;
    }
  } catch (e) {alert("DMT Startup() failed:\n" + e.name + "\n" + e.message);}

  // Setup listeners so that the command/info bar can be updated.
  try {
    // Need to change the button enables and infoBar data as different
    // downloads are selected.
    gDownloadsView.addEventListener("select", dmtOnDownloadSelect, false);
    dmtSavedDownloadsView = gDownloadsView;
    dmtDownloadManager.addEventListener("click", dmtOnDownloadClick, false);

    // This listens for window resize events, so that the buttons can be switched
    // between wide and narrow.
    addEventListener("resize", dmtOnDownloadResize, false);

    // This listens for download state changes and updates the button enables.
    // For example, you can't launch a file that is still downloading, but you
    // can once the download is complete.
    dmtDownloadListener = new dmtDownloadProgressListener();
    gDownloadManager.addListener(dmtDownloadListener);

    // Listens for changes to the preferences (mainly on the options popup)
    // and immediately redraws the screen to reflect the changes.
    dmtPreferencesService.QueryInterface(Components.interfaces.nsIPrefBranch2);
    dmtPreferencesService.addObserver("extensions.downloadmgr.", dmtPrefObserver, false);
  } catch (e) {alert("DMT Startup() failed:\n" + e.name + "\n" + e.message);}
}


////////////////////////////////////////////////////////////////////////////////
// Cleanup the extended features added by DMT and then shutdown the original
// download manger being tweaked.
function dmtShutdown()
{
  // Cleanup listeners.
  try {
    dmtPreferencesService.QueryInterface(Components.interfaces.nsIPrefBranch2);
    dmtPreferencesService.removeObserver("extensions.downloadmgr.", dmtPrefObserver);

    gDownloadManager.removeListener(dmtDownloadListener);

    removeEventListener("resize", dmtOnDownloadResize, false);

    dmtDownloadManager.removeEventListener("click", dmtOnDownloadClick, false);
    gDownloadsView.removeEventListener("select", dmtOnDownloadSelect, false);
    dmtSavedDownloadsView.removeEventListener("select", dmtOnDownloadSelect, false);
  } catch (e) {}

  // Shutdown() is inside mozapps/content/downloads/downloads.js, the download
  // manager built into FireFox.
  Shutdown();
}

////////////////////////////////////////////////////////////////////////////////
// Called when the options button is pressed. Displays the options dialog.
function dmtShowOptions()
{
  openDialog("chrome://downloadmgr/content/options/dmtOptions.xul", "", "chrome,modal");
}

////////////////////////////////////////////////////////////////////////////////
// Loops through all the selected downloads and reloads the associated file.
// Only done if file doesn't exists. Uses retry if load failed or was canceled
// and does something special if download finished but file was deleted.
function dmtReloadFiles()
{
  // Do the file reload for each download in the list of selected downloads.
  for (let i = 0; i < gDownloadsView.selectedItems.length; i++)
  {
    try
    {
      // Get a file object from the file name.
      var thisDownload = gDownloadsView.selectedItems[i];
      var fileName = thisDownload.getAttribute("file");
      var filePath = thisDownload.getAttribute("path");
      var fileObj = getLocalFileFromNativePathOrUrl(fileName);
      var dlState = parseInt(thisDownload.getAttribute("state"));

      // Make sure file doesn't exist before trying to redownload it.
      if (!fileObj.exists())
      {
        switch (dlState) {
        case nsIDM.DOWNLOAD_FAILED:
        case nsIDM.DOWNLOAD_CANCELED:
          // Use the standard retry if possible.
          gDownloadViewController.doCommand('cmd_retry', thisDownload);
          break;

        case nsIDM.DOWNLOAD_FINISHED:
          // Can't use retry, have to create a new download.
          const nsIWBP = Ci.nsIWebBrowserPersist;
          var ios = Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService);
          var fromURI = ios.newURI(thisDownload.getAttribute('uri'), null, null);
          var toURI = ios.newFileURI(fileObj);
          var persist = null;
          var dl = null;

          // Remove the download from the list, a new one will be created below.
          gDownloadViewController.doCommand('cmd_removeFromList', thisDownload);

          // Create a persist object. This controls and monitors the download.
          persist = Cc["@mozilla.org/embedding/browser/nsWebBrowserPersist;1"]
                    .createInstance(Ci.nsIWebBrowserPersist);
          persist.persistFlags = nsIWBP.PERSIST_FLAGS_REPLACE_EXISTING_FILES |
                                 nsIWBP.PERSIST_FLAGS_BYPASS_CACHE |
                                 nsIWBP.PERSIST_FLAGS_AUTODETECT_APPLY_CONVERSION;

          // Create a new download.
          dl = gDownloadManager.addDownload(
                            Ci.nsIDownloadManager.DOWNLOAD_TYPE_DOWNLOAD,
                            fromURI, toURI, null, null, 0, null, persist);

          // Set the progress listener to the download object and then start
          // the download.
          persist.progressListener = dl.QueryInterface(Ci.nsIWebProgressListener);
          persist.saveURI(dl.source, null, null, null, null, dl.targetFile);

          // This replaces all of the above, but doesn't behave as well.
          //window.location = thisDownload.getAttribute('uri');
          //gDownloadViewController.doCommand('cmd_removeFromList', thisDownload);
          break;

        default:
          break;
        }
      }
    } catch (e) {}
  }
}

////////////////////////////////////////////////////////////////////////////////
// Loops through all the selected downloads and deletes the associated file.
// Only done if file exists and user confirms the delete.
function dmtDeleteFiles()
{
  var fileWasDeleted = false;
  var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].
                      getService(Components.interfaces.nsIPromptService);

  // Do the file delete for each download in the list of selected downloads.
  for (let i = 0; i < gDownloadsView.selectedItems.length; i++)
  {
    try
    {
      // Get a file object from the file name.
      var fileName = gDownloadsView.selectedItems[i].getAttribute("file")
      var filePath = gDownloadsView.selectedItems[i].getAttribute("path");
      var fileObj = getLocalFileFromNativePathOrUrl(fileName);

      // Make sure file exists before trying to delete it.
      if (fileObj.exists())
      {
        var check = {value: false};
        var flags = promptService.BUTTON_POS_0 * promptService.BUTTON_TITLE_YES +
                    promptService.BUTTON_POS_1 * promptService.BUTTON_TITLE_CANCEL +
                    promptService.BUTTON_POS_2 * promptService.BUTTON_TITLE_NO;
        var doDelete = 0;

        // Prompt the user to make sure they want to delete each file.
        doDelete = promptService.confirmEx(null, dmtProjectName,
                                           dmtDeleteFileButtonLabel + "\n" + filePath,
                                           flags, "", "", "", null, check);
        if (doDelete == 0)
        {
          // Selected yes.
          fileObj.remove(false);
          fileWasDeleted = true;
        }
        else if (doDelete == 1)
        {
          // Selected cancel.
          break;
        }
      }
    } catch (e) {}
  }

  // Update button greyouts, if a file was deleted.
  if (fileWasDeleted)
  {
    var dlState = 0;

    // Get parameter for dmtUpdateToolInfoBar().
    if ((gDownloadsView.selectedCount > 0) && gDownloadsView.selectedItem)
    {
      dlState = parseInt(gDownloadsView.selectedItem.getAttribute("state"));
    }
    dmtUpdateToolInfoBar(dlState);
  }
}

////////////////////////////////////////////////////////////////////////////////
// The dmtDownloadProgressListener class is used to update the toolbar and
// infobar for downloads in progress.
// This class implements the nsIDownloadProgressListener interface.
function dmtDownloadProgressListener() {}

dmtDownloadProgressListener.prototype = {
  QueryInterface: XPCOMUtils.generateQI([Ci.nsIDownloadProgressListener]),

  onDownloadStateChange: function dlPL_onDownloadStateChange(aState, aDownload)
  {
    // One of the downloads changed state. If the changed download is the same
    // as the selected download then the toolbar needs to be updated.
    if (gDownloadsView.selectedItem == getDownload(aDownload.id))
    {
      // Update the entire toolbar and infobar, mainly the button greyouts.
      dmtUpdateToolInfoBar(aDownload.state);
    }
  },
};
